/*
Copyright (c) 2004, Devin Watson <dmwatson@comcast.net> 
and 2010 Micke Nordin <mickewiki@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mnenc nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Devin Watson BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* */

#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <cstdio>
#include <cstdlib>
#include <string>
#include <cerrno>
#include <fstream>
#include <iostream>
#include "mnenc.hpp"
#include "password.hpp"
#include "mnencd.hpp"

int main(int argc, char** argv) {
	/* Our process ID and Session ID */
	pid_t pid, sid;
	
	/* Fork off the parent process */
	pid = fork();
	if (pid < 0) {
		exit(EXIT_FAILURE);
	}
	/* If we got a good PID, then
	   we can exit the parent process. */
	if (pid > 0) {
		exit(EXIT_SUCCESS);
	}

	/* Change the file mode mask */
	umask(0);
		
	/* Open any logs here */	
	std::string name, dirname;
	name = getenv("USER");
	dirname = "/home/" + name + "/.mnenc/";
	if(mkdir(dirname.c_str(), S_IRWXU | S_IRWXG | S_IRWXO ) < 0) {
		if(errno != EEXIST) {
			exit(EXIT_FAILURE);
		}
	}
	/*This is log stuff */
	std::string logname = "/home/" + name + "/.mnenc/log";
	std::ofstream log; //Log stream
	log.open(logname.c_str(), ios::app); // open log
	time_t rawtime; //Time stuff here
	struct tm * timeinfo;
	time ( &rawtime );
	timeinfo = localtime ( &rawtime );
	std::string current_time = asctime (timeinfo); //Holds current time
	
	/* Create a new SID for the child process */
	sid = setsid();
	if (sid < 0) {
		log << current_time << ": Unable to create a new SID for the child process\n";
		exit(EXIT_FAILURE);
	}
	
	/* Change the current working directory */
	if ((chdir("/tmp/")) < 0) {
		log << current_time << ": Could not change to /tmp\n";
		exit(EXIT_FAILURE);
	}
	
	/* Close out the standard file descriptors */
	close(STDIN_FILENO);
	close(STDOUT_FILENO);
	close(STDERR_FILENO);
	
	/* Daemon-specific initialization goes here */
	mkfifo("./mnencdfifo", 0777); //This is the fifo for communications
	signal(SIGTERM, term); // register a SIGTERM handler
	log.close(); //Closing log 
	
	/* The Big Loop */
	while (1) {
		std::string request = m_read();
		std::string reply = do_something(request);
		m_send(reply);
		sleep(1); /* wait 1 second*/
	}
	exit(EXIT_SUCCESS);
}
